module Skeksis
  class IR
    Header = <<~HTML
    <html>
    <head>
    <title>Gembridge</title>
    </head>
    <body>
    HTML

    Footer = <<~HTML
    </body>
    </html>
    HTML

    def htmlize
      build_html_doc
    end

    private 
    def build_html_doc
      content = self.parse_ir.join("\n")
      Header + "#{content}\n" + Footer
    end

    def parse_ir
      html = self.map do |entry|
        content = lambda { |entry|
          unless entry[:content].nil? then return entry[:content][0] end
        }

        text = content.call(entry)

        case entry[:type]
        when :blank
          "<br/>"
        when :header
          level = entry[:level]
          "<h#{level}>#{text}</h#{level}>"
        when :list
          # TODO: Does not handle lists properly, only one <ul> tag per group of lists needs to be
          # created instead
          "<ul><li>#{text}</li></ul>"
        when :quote
          "<pre>#{text}</pre>"
        when :uri
          uri = entry[:uri]
          text = entry[:text]
          "<a href=\"#{uri}\">#{text}</a>"
        when :verbatim
          "<pre>" + entry[:content].join("\n") + "</pre>"
        when :text
          "<p>#{text}</p>"
        end
      end

      html
    end
  end
end
